HideFrames = CreateFrame("Frame", UIParent)



VIEWED_SPELL_BOOK = 2;
VIEWED_SPELL_BOOK_NUM_SPELLS = nil;
SHOW_BOBUI_ABILITY_TAB_AFTER_COMBAT = false
SPELL_BOOK_TYPE = "spells"
CURRENT_FLYOUT = nil

VIEWED_TAB_ID = nil

SPELL_BOOK_PROFESSIONS = {}

function getRecommendedScale()
	local scale = string.match(GetCVar("gxWindowedResolution" ), "%d+x(%d+)");

	return 768/scale
end

BobUI_Settings_Recommended = {
	["BorderSize"] = 2,
	["SeparatorSize"] = 1,
	["SpellIconSize"] = 36,
	["TabIconSize"] = 36,
	["Scaling"] = getRecommendedScale(),
	["FontSizeHeader"] = 18,
	["FontSizeBody"] = 14,
	["BackgroundColor"] = {
		["r"] = 0, 
		["g"] = 0, 
		["b"] = 0, 
		["a"] = 0.9
	},
	["SeparatorColor"] = {
		["r"] = 0.1, 
		["g"] = 0.1, 
		["b"] = 0.1, 
		["a"] = 1
	},
	["TextColor"] = {
		["r"] = 0.8, 
		["g"] = 0.8, 
		["b"] = 0.8, 
		["a"] = 1
	},
}

if BobUI_Settings == nil then 
	BobUI_Settings = {}
	for k,v in pairs(BobUI_Settings_Recommended) do
		BobUI_Settings[k] = v
	end
end

function editBox_OnKeyUp(self, event)
	if event == "ESCAPE" then
		self:ClearFocus()
	end
end

function editBoxKeyDown(self, event)
	local editBoxText = self:GetText():gsub('[^0-9+.]', '')

	if event == "ENTER" then
		if editBoxText == "" or ((self.settingName == "FontSizeHeader" or self.settingName == "FontSizeBody") and tonumber(editBoxText) < 1) then
			if self.settingName == "Scaling" then 
				editBoxText = getRecommendedScale()
			else
				editBoxText = BobUI_Settings_Recommended[self.settingName]
			end
		end
		
		self.settingValue = editBoxText
		self:SetText(editBoxText)
		applySetting(self)
		self:ClearFocus()


		if BobUI_AbilityTab:IsShown() then BobUI_AbilityTab:Hide(); BobUI_AbilityTab:Show() end
	end
end

function applySetting(self) 
	BobUI_Settings[self.settingName] = self.settingValue

	if self.settingName == "Scaling" then
		BobUI_AbilityTab:SetScale(BobUI_Settings["Scaling"])
	end
end

function getBackDropBorderColorFromClassColors(active) 
	-- if not active, then highlighted. Difference, is alpha decreased on active. 

	local className, classFileName, classID = UnitClass("player")
	local classColorR, classColorG, classColorB = RAID_CLASS_COLORS[classFileName]:GetRGB()
	local classColorA = 1

	if active == true then
		classColorA = 0.4
	end

	return classColorR, classColorG, classColorB, classColorA
end

function BobUI_FormatProfession(PROFESSION_INDEX, PROFESSION_ID)
	if PROFESSION_ID == nil then SPELL_BOOK_PROFESSIONS[PROFESSION_INDEX] = nil; return end 
	local name, texture, rank, maxRank, numSpells, spelloffset, skillLine, rankModifier, specializationIndex, specializationOffset, skillLineName = GetProfessionInfo(PROFESSION_ID);
	
	SPELL_BOOK_PROFESSIONS[PROFESSION_INDEX] = {
		["name"] = name,
		["texture"] = texture,
		["rank"] = rank,
		["maxRank"] = maxRank,
		["numSpells"] = numSpells,
		["spelloffset"] = spelloffset,
		["skillLine"] = skillLine,
		["rankModifier"] = rankModifier,
		["specializationIndex"] = specializationIndex,
		["specializationOffset"] = specializationOffset,
		["skillLineName"] = skillLineName
	}

	if (PROFESSION_INDEX == 1) then
		BobUI_specButtons.specButton7.specIcon:SetTexture(texture);
	end
end


function GetNumSpellsForSpellBook(SPELL_BOOK)
	
	local name, texture, offset, numSlots, isGuild, offSpecID, shouldHide, specID = GetSpellTabInfo(SPELL_BOOK);

	if SPELL_BOOK == 6 then
		numSlots = select(1, HasPetSpells())
	end

	if SPELL_BOOK == 7 then -- professions
		local prof1, prof2, arch, fish, cook = GetProfessions();

		BobUI_FormatProfession(1, prof1);
		BobUI_FormatProfession(2, prof2);
		BobUI_FormatProfession(3, cook);
		BobUI_FormatProfession(4, fish);
		BobUI_FormatProfession(5, arch);
		
		numSlots = 0

		for i=1,5 do
			if (SPELL_BOOK_PROFESSIONS[i] ~= nil) then
				numSlots = numSlots + SPELL_BOOK_PROFESSIONS[i]["numSpells"]
			end
		end

	end


	return numSlots
end


function modifiedClick(spellIndex, button)
	if ( IsModifiedClick() ) then
		if ( IsModifiedClick("CHATLINK") ) then
			if ( MacroFrameText and MacroFrameText:HasFocus() ) then
				local spellName, subSpellName = GetSpellBookItemName(spellIndex, SPELL_BOOK_TYPE);
				if ( spellName and not IsPassiveSpell(spellIndex, SPELL_BOOK_TYPE) ) then
					if ( subSpellName and (strlen(subSpellName) > 0) ) then
						ChatEdit_InsertLink(spellName.."("..subSpellName..")");
					else
						ChatEdit_InsertLink(spellName);
					end
				end
				return;
			else
				local tradeSkillLink, tradeSkillSpellID = GetSpellTradeSkillLink(spellIndex, SPELL_BOOK_TYPE);
				if ( tradeSkillSpellID ) then
					ChatEdit_InsertLink(tradeSkillLink);
				else
					local spellLink = GetSpellLink(spellIndex, SPELL_BOOK_TYPE);
					ChatEdit_InsertLink(spellLink);
				end
				return;
			end
		end
	end
end

function getProfessionOffset(li)
	local returned = 0
	local spellCount = 0
	local numSpells = 0
	local isFirstOfProfession = false

	local learnedProfessionAbilityLocations = {}

	for i=1,5 do
		if SPELL_BOOK_PROFESSIONS[i] ~= nil then
			spellCount = spellCount + SPELL_BOOK_PROFESSIONS[i]["numSpells"]
			local numSpellsForProfession = SPELL_BOOK_PROFESSIONS[i]["numSpells"]

			for i1=1,numSpellsForProfession do
				table.insert(learnedProfessionAbilityLocations, i)
			end
		end
	end


	if SPELL_BOOK_PROFESSIONS[learnedProfessionAbilityLocations[li]] ~= nil then
		returned = SPELL_BOOK_PROFESSIONS[learnedProfessionAbilityLocations[li]]["spelloffset"]
		numSpells = SPELL_BOOK_PROFESSIONS[learnedProfessionAbilityLocations[li]]["numSpells"]

		if learnedProfessionAbilityLocations[li-1] ~= nil then
			if learnedProfessionAbilityLocations[li-1] ~= learnedProfessionAbilityLocations[li] then
				isFirstOfProfession = true
			end
		else
			isFirstOfProfession = true
		end
	end

	return returned - li + 1 + (isFirstOfProfession and 0 or 1), isFirstOfProfession, learnedProfessionAbilityLocations[li]
end

function UpdateCooldown(self, slot, slotType)
	
	if (slot) then

		local start, duration, enable, modRate = GetSpellCooldown(slot, slotType);

		if (self.cooldown and start and duration) then
			if (enable) then
				self.cooldown:Hide();
				self.cooldown.coolDownTimer:Hide();
			else
				self.cooldown:Show();
				self.cooldown.coolDownTimer:SetText((duration + start) - GetTime())
				self.cooldown.coolDownTimer:Show();
			end

			CooldownFrame_Set(self.cooldown, start, duration, enable, true, modRate);
		else
			self.cooldown:Hide();
			self.cooldown.coolDownTimer:Hide();
		end
	end
end


function BobUI_spellBookItemData(i)
	local offset = 0;
	local numFlyoutSpells = 0
	local professionIndex = 0
			
	if SPELL_BOOK_TYPE == "profession" then
		offset, isFirstOfProfession, professionIndex = getProfessionOffset(i)
	elseif SPELL_BOOK_TYPE == "spell" then
		_, _, offset = GetSpellTabInfo(VIEWED_SPELL_BOOK);
	end

	local texture = GetSpellBookItemTexture(i+offset, SPELL_BOOK_TYPE);
	local spellType, actionID = GetSpellBookItemInfo(i+offset, SPELL_BOOK_TYPE);
	local spellName, subSpellName, spellID = GetSpellBookItemName(i+offset, SPELL_BOOK_TYPE)

	local isOnActionbar = (VIEWED_SPELL_BOOK ~= 2 and true or C_ActionBar.IsOnBarOrSpecialBar(actionID))
	if (SPELL_BOOK_TYPE == "pet") then
		isOnActionbar = C_ActionBar.HasPetActionButtons(actionID) or C_ActionBar.HasPetActionPetBarIndices(actionID);
	end

	local isPassive
	if (SPELL_BOOK_TYPE == "profession") then 
		isPassive = false 
	else 
		isPassive = IsPassiveSpell(i+offset, SPELL_BOOK_TYPE) 
	end

	if spellType == "FLYOUT" then
		numFlyoutSpells = select(3, GetFlyoutInfo(actionID))
	end
	local spellLevel = GetSpellAvailableLevel(i+offset, SPELL_BOOK_TYPE)

	local autoCastAllowed, autoCastEnabled = GetSpellAutocast(i+offset, SPELL_BOOK_TYPE)

	return offset, isFirstOfProfession, isPassive, isOnActionbar, texture, spellType, numFlyoutSpells, actionID, spellName, subSpellName, spellID, spellLevel, autoCastAllowed, autoCastEnabled, professionIndex
end


function AssignScriptsToButtons(button, i, offset, missingOnActionBar, SPELL_BOOK_TYPE, spellID, spellName, actionID, autoCastAllowed)
	button:SetScript("OnLeave", function()
		GameTooltip:Hide()
	end)
	button:SetScript("OnEnter", function()
		GameTooltip:Hide()
		GameTooltip:ClearAllPoints()
		GameTooltip:SetOwner(button, "ANCHOR_RIGHT")
		
		if spellID then
			GameTooltip:SetSpellByID(spellID)
		else
			GameTooltip:SetSpellBookItem(i+offset, SPELL_BOOK_TYPE)
		end

		GameTooltip:AddLine(missingOnActionBar and SPELLBOOK_SPELL_NOT_ON_ACTION_BAR or "",LIGHTBLUE_FONT_COLOR.r, LIGHTBLUE_FONT_COLOR.g, LIGHTBLUE_FONT_COLOR.b);
		GameTooltip:Show()
	end)

	button:SetScript("OnDragStart", function()
		if spellID and SPELL_BOOK_TYPE ~= "pet" then
			if spellID == 53271 or spellID == 62305 or spellID == 272682 or spellID == 264735 or spellID == 281195 or spellID == 264667 or spellID == 272679 then
				PickupSpell(272651)
			else
				PickupSpell(spellID)
			end
		else
			if not IsPassiveSpell(i+offset, SPELL_BOOK_TYPE) then
				PickupSpellBookItem(i+offset, SPELL_BOOK_TYPE)
			end
		end
	end)

	button:RegisterEvent("SPELLS_CHANGED")
	button:RegisterEvent("SPELL_UPDATE_COOLDOWN")
	button:SetScript("OnEvent", function(self, event, ...)
		if event == "SPELL_UPDATE_COOLDOWN" or event == "SPELLS_CHANGED" then
			UpdateCooldown(self, i+offset, SPELL_BOOK_TYPE)
		end
	end)

	if SPELL_BOOK_TYPE == "pet" then 
		if (spellID ~= nil) then
			if autoCastAllowed then
				button:RegisterForClicks("AnyUp")
				button:SetAttribute("type2", "macro");
				button:SetAttribute("macrotext", "/petautocasttoggle "..spellName);
			end
				button:SetAttribute("type1", "spell");
				button:SetAttribute("spell", spellID);
		else
			if spellName == "Defensive" then
				button:SetAttribute("type1", "macro");
				button:SetAttribute("macrotext", "/petdefensive"); -- doesn't work, idk how to fix.
			elseif spellName == "Move To" then
				button:SetAttribute("type1", "macro");
				button:SetAttribute("macrotext", "/petmoveto");
			else
				
				button:SetAttribute("type1", "macro");
				button:SetAttribute("macrotext", "/pet"..spellName);
			end
				
		end
	else 
		button:SetAttribute("type1", "spell");
		button:SetAttribute("spell", actionID);
		button:SetAttribute("target", "target");
	end
end



function toggleAutocastShine(button, bIndex)
	local autoCastableTexture = _G[button:GetName().."AutoCastable"];
	local petAutoCastShine = _G[button:GetName().."Shine"];
	local autoCastAllowed, autoCastEnabled = GetSpellAutocast(bIndex, SPELL_BOOK_TYPE)


	if ( autoCastAllowed ) then
		autoCastableTexture:Show();
	else
		autoCastableTexture:Hide();
	end
	
	if autoCastEnabled then
		AutoCastShine_AutoCastStart(petAutoCastShine)
	else
		AutoCastShine_AutoCastStop(petAutoCastShine)
	end
end

function updateSpellButtons()

	BobTabPage1:SetWidth(9 * BobUI_Settings["SpellIconSize"] + (8*4))
	--BobTabPage2:SetWidth(BobTabPage1:GetWidth())
	BobTabPage1Actives:SetWidth(BobTabPage1:GetWidth())
	BobTabPage1Passives:SetWidth(BobTabPage1:GetWidth())
	BobTabPage1FlyoutButtons:SetWidth(BobTabPage1:GetWidth())
	BobTabPage1PetControlsAndFlyouts:SetWidth(BobTabPage1:GetWidth())

	BobTabPage1Actives:SetPoint("TOPLEFT", BobTabPage1Actives:GetParent(), "TOPLEFT", 0, -1 * BobUI_Settings["FontSizeBody"])
	BobTabPage1Actives.category:SetPoint("TOPLEFT", BobTabPage1Actives, "TOPLEFT", 0, BobUI_Settings["FontSizeBody"])

	BobTabPage1Passives:SetPoint("TOPLEFT", BobTabPage1Actives, "BOTTOMLEFT", 0, (-1 * BobUI_Settings["FontSizeBody"]) - 10)
	BobTabPage1Passives.category:SetPoint("TOPLEFT", BobTabPage1Passives, "TOPLEFT", 0, BobUI_Settings["FontSizeBody"])

	BobUI_PlayerTalentFrameTalentsTalents:SetPoint("TOPLEFT", BobUI_PlayerTalentFrameTalentsTalents:GetParent().borderLeft, "TOPLEFT", 10, (-1 * BobUI_Settings["FontSizeBody"]) - 5)
	BobUI_PlayerTalentFrameTalentsTalents.category:SetPoint("TOPLEFT", BobUI_PlayerTalentFrameTalentsTalents, "TOPLEFT", BobUI_Settings["SeparatorSize"], BobUI_Settings["FontSizeBody"])
	BobUI_PlayerTalentFrameTalentsTalentRow1:SetPoint("LEFT", BobUI_PlayerTalentFrameTalentsTalents, "LEFT", BobUI_Settings["SeparatorSize"], 0)

	BobUI_PlayerTalentFrameTalentsBobUI_PvpTalentFramePVPTalents:SetPoint("TOPLEFT", BobUI_PlayerTalentFrameTalentsBobUI_PvpTalentFramePVPTalents:GetParent(), "TOPLEFT", 10, (-1 * BobUI_Settings["FontSizeBody"]))
	BobUI_PlayerTalentFrameTalentsBobUI_PvpTalentFramePVPTalents.category:SetPoint("TOPLEFT", BobUI_PlayerTalentFrameTalentsBobUI_PvpTalentFramePVPTalents, "TOPLEFT", 0, BobUI_Settings["FontSizeBody"])

	BobUI_PlayerTalentFrameTalentsEssencesTitle:SetPoint("TOPLEFT", BobUI_PlayerTalentFrameTalentsEssences, "TOPLEFT", 10, (-1 * BobUI_Settings["FontSizeBody"]))
	BobUI_PlayerTalentFrameTalentsEssencesTitle.category:SetPoint("TOPLEFT", BobUI_PlayerTalentFrameTalentsEssencesTitle, "TOPLEFT", 0, BobUI_Settings["FontSizeBody"])

	--BobTabPage1Passives:SetWidth(BobTabPage1:GetWidth())
	
	BobTabPage1Actives.category:SetFont("Fonts\\ARIALN.ttf", BobUI_Settings["FontSizeBody"])
	BobTabPage1Passives.category:SetFont("Fonts\\ARIALN.ttf", BobUI_Settings["FontSizeBody"])
	BobUI_PlayerTalentFrameTalentsTalents.category:SetFont("Fonts\\ARIALN.ttf", BobUI_Settings["FontSizeBody"])
	BobUI_PlayerTalentFrameTalentsBobUI_PvpTalentFramePVPTalents.category:SetFont("Fonts\\ARIALN.ttf", BobUI_Settings["FontSizeBody"])
	BobUI_PlayerTalentFrameTalentsEssencesTitle.category:SetFont("Fonts\\ARIALN.ttf", BobUI_Settings["FontSizeBody"])

	
	--BobTabPage1FlyoutButtons.category:SetFont("Fonts\\ARIALN.ttf", BobUI_Settings["FontSizeBody"])
	--BobTabPage1PetControlsAndFlyouts.category:SetFont("Fonts\\ARIALN.ttf", BobUI_Settings["FontSizeBody"])



	if VIEWED_SPELL_BOOK == 6 then 
		SPELL_BOOK_TYPE = "pet"
	elseif VIEWED_SPELL_BOOK == 7 then 
		SPELL_BOOK_TYPE = "profession"
	else
		SPELL_BOOK_TYPE = "spell"
	end



	local buttons;
	local newLineAt = 9;

	if ( BobTabPage1.buttons ) then
		buttons = BobTabPage1.buttons;
	else
		return
	end


	local j = 1
	local k = 1
	local l = 1
	local m = 1
	local vCi= 1
	local unlearnProfession = 0

	local startOfLinePassives = nil
	local startOfLineActives = nil
	local startOfLinePetControls = nil
	local startOfLineFlyouts = nil

	local lastPassive = nil
	local lastActive = nil
	local lastPetControl = nil
	local lastFlyout = nil

	local LastFirstOfProfession = nil

	BobTabPage1Passives:SetHeight(0)
	BobTabPage1Actives:SetHeight(0)
	BobTabPage1PetControlsAndFlyouts:SetHeight(0)
	BobTabPage1FlyoutButtons:SetHeight(40)

	for i = 1, #buttons do 
		local button = buttons[i];
		button:Hide(); 
		if i <= VIEWED_SPELL_BOOK_NUM_SPELLS then
			local offset, isFirstOfProfession, isPassive, isOnActionbar, texture, spellType, numFlyoutSpells, actionID, spellName, subSpellName, spellID, spellLevel, autoCastAllowed, autoCastEnabled, professionIndex = BobUI_spellBookItemData(i)

			toggleAutocastShine(button, i+offset)

			if (spellType == "FLYOUT") then
				button.FlyoutArrow:Show()
				SetClampedTextureRotation(button.FlyoutArrow, 180);
				isOnActionbar = true
			else
				button.FlyoutArrow:Hide()
			end

			local missingOnActionBar = ((isOnActionbar ~= true) and (isPassive ~= true))
			
			local isDisabled = spellID and C_SpellBook.IsSpellDisabled(spellID);
			
			--- change this at some point... I guess

			_G[button:GetName().."RequiredLevelString"]:SetFont("Fonts\\ARIALN.ttf", BobUI_Settings["FontSizeBody"], "OUTLINE")
			_G[button:GetName().."RequiredLevelString"]:SetTextColor(0.8,0.8,0.8,1)

			if not (spellType == "FUTURESPELL") and not isDisabled then
					button.TrainBook:Hide()
					_G[button:GetName().."IconTexture"]:SetDesaturated(false)
					_G[button:GetName().."RequiredLevelString"]:Hide()				
			else
				if spellType == "FUTURESPELL" then
					missingOnActionBar = false

					if (spellLevel and spellLevel > UnitLevel("player") or isDisabled) then
						_G[button:GetName().."RequiredLevelString"]:SetText("Lv "..spellLevel)
						_G[button:GetName().."RequiredLevelString"]:Show()
					else
						button.TrainBook:Show()
					end

					_G[button:GetName().."IconTexture"]:SetDesaturated(true)
				end
			end
			if missingOnActionBar then button.SpellHighlightTexture:Show() else button.SpellHighlightTexture:Hide() end

			local newLinePassives = (math.fmod(j-1,newLineAt) == 0)
			local newLineActives = (math.fmod(k-1,newLineAt) == 0)
			local newLinePetControls = (math.fmod(l-1,newLineAt) == 0)
			local newLineFlyouts = (math.fmod(m-1,newLineAt) == 0)

			button:ClearAllPoints()
			if spellType == "FLYOUT" then
				if m - 1 == 0 then
					button:SetPoint("TOPLEFT", BobTabPage1PetControlsAndFlyouts, "TOPLEFT", 0, 0);
				else
					if (newLineFlyouts) then
						button:SetPoint("TOPLEFT", startOfLineFlyouts, "BOTTOMLEFT", 0,-4);
					end
				end
				if newLineFlyouts then
					startOfLineFlyouts = buttons[i]
					BobTabPage1PetControlsAndFlyouts:SetHeight(BobTabPage1PetControlsAndFlyouts:GetHeight() + BobUI_Settings["SpellIconSize"] + 4)
				else
					button:SetPoint("LEFT", lastFlyout or startOfLineFlyouts, "RIGHT", 4, 0);
				end
				lastFlyout = buttons[i]

			else
				if isPassive then

					if j - 1 == 0 then
						button:SetPoint("TOPLEFT", BobTabPage1Passives, "TOPLEFT", 0, 0);
					else
						if (newLinePassives) then
							button:SetPoint("TOPLEFT", startOfLinePassives, "BOTTOMLEFT", 0,-4);
						end
					end
					if newLinePassives then
						startOfLinePassives = buttons[i]
						BobTabPage1Passives:SetHeight(BobTabPage1Passives:GetHeight() + BobUI_Settings["SpellIconSize"] + 4)
					else
						button:SetPoint("LEFT", lastPassive or startOfLinePassives, "RIGHT", 4, 0);
					end
					lastPassive = buttons[i]

				else
					if spellID == nil and SPELL_BOOK_TYPE == "pet" then
						if l - 1 == 0 then
							button:SetPoint("TOPLEFT", BobTabPage1PetControlsAndFlyouts, "TOPLEFT", 0, 0);
						else
							if (newLinePetControls) then
								button:SetPoint("TOPLEFT", startOfLinePetControls, "BOTTOMLEFT", 0,-4);
							end
						end
						if newLinePetControls then
							startOfLinePetControls = buttons[i]
							BobTabPage1PetControlsAndFlyouts:SetHeight(BobTabPage1PetControlsAndFlyouts:GetHeight() + BobUI_Settings["SpellIconSize"] + 4)
						else
							button:SetPoint("LEFT", lastPetControl or startOfLinePetControls, "RIGHT", 4, 0);
						end
						lastPetControl = buttons[i]
					else
						if k - 1 == 0 then
							button:SetPoint("TOPLEFT", BobTabPage1Actives, "TOPLEFT", 0, 0);
						else
							if (newLineActives) then
								button:SetPoint("TOPLEFT", startOfLineActives, "BOTTOMLEFT", 0,-4);
							end
						end
						if newLineActives then
							startOfLineActives = buttons[i]
							BobTabPage1Actives:SetHeight(BobTabPage1Actives:GetHeight() + BobUI_Settings["SpellIconSize"] + 4)
						else
							button:SetPoint("LEFT", lastActive or startOfLineActives, "RIGHT", 4, 0);
						end
						lastActive = buttons[i]
					end

				end
			end

			AssignScriptsToButtons(button, i, offset, missingOnActionBar, SPELL_BOOK_TYPE, spellID, spellName, actionID, autoCastAllowed)
			
			button:SetScript("OnMouseUp", function(self, mBtn)
				if ( IsModifiedClick() ) then
					modifiedClick(i+offset, mBtn)
				else
					if (spellType == "FLYOUT") then -----------
						local startPt = VIEWED_SPELL_BOOK_NUM_SPELLS + 1
						local previousButton = nil
						local newLineBtn = nil
						local btnCount = 1
						if CURRENT_FLYOUT == actionID then
							for x1 = startPt, #buttons do 
								buttons[x1]:Hide()
							end
							CURRENT_FLYOUT = nil
						else
							CURRENT_FLYOUT = actionID

							for x1 = startPt, #buttons do 
								local flyoutButton = buttons[x1]
								flyoutButton:ClearAllPoints()

								if x1 > startPt + numFlyoutSpells - 1 then
									flyoutButton:Hide()
								else

									local flyoutSpellID, flyoutOverrideSpellID, flyoutIsKnown, flyoutSpellName, flyoutSlotSpecID = GetFlyoutSlotInfo(actionID, (x1-startPt)+1);
									

									if flyoutIsKnown then
										if previousButton == nil then
											flyoutButton:SetPoint("TOPLEFT", BobTabPage1FlyoutButtons, "TOPLEFT", 0, 0);
											newLineBtn = flyoutButton
										elseif btnCount == 8 then
											flyoutButton:SetPoint("TOPLEFT", newLineBtn, "BOTTOMLEFT", 0, -2);
											btnCount = 1
											newLineBtn = flyoutButton
										else
											flyoutButton:SetPoint("LEFT", previousButton, "RIGHT", 2, 0);
										end
										
										AssignScriptsToButtons(flyoutButton, 0, 0, false, nil, flyoutSpellID, flyoutSpellName, flyoutSpellID, autoCastAllowed)

										previousButton = buttons[x1]
										btnCount = btnCount + 1

										_G[flyoutButton:GetName().."IconTexture"]:SetTexture(GetSpellBookItemTexture(flyoutSpellName))
										_G[flyoutButton:GetName().."IconTexture"]:Show()
										flyoutButton:SetSize(30,30)
										flyoutButton:Show()
									else
										flyoutButton:Hide()
									end
								end
							end
						end
					elseif (mBtn == "RightButton" and SPELL_BOOK_TYPE == "pet") then
						
						wait(0.2, toggleAutocastShine, button, i+offset)
						updateSpellButtons()
					end
				end
			end)
			

			
			
			_G[button:GetName().."IconTexture"]:SetTexture(texture)
			_G[button:GetName().."IconTexture"]:Show()
			button.cooldown:Hide();
			button.cooldown.coolDownTimer:Hide();
			button.unlearn:Hide()
			button.profText:Hide()
			button.AbilityHighlight:Hide();
			button.AbilityHighlightAnim:Stop();
			button:SetSize(BobUI_Settings["SpellIconSize"], BobUI_Settings["SpellIconSize"])




			if SPELL_BOOK_TYPE == "profession" then
				if isFirstOfProfession then
					
					if (LastFirstOfProfession == nil) then
						button:SetPoint("TOPLEFT", BobTabPage1Actives, "TOPLEFT", 0, 0);
					else
						button:SetPoint("TOPLEFT", LastFirstOfProfession, "BOTTOMLEFT", 0, -4);
					end

					button:SetSize(BobUI_Settings["SpellIconSize"] * 1.25, BobUI_Settings["SpellIconSize"] * 1.25)
					button.profText:SetFont("Fonts\\ARIALN.ttf", BobUI_Settings["FontSizeBody"])
					button.profText:SetText(SPELL_BOOK_PROFESSIONS[professionIndex]["name"] .. "\n" .. SPELL_BOOK_PROFESSIONS[professionIndex]["rank"] .. "/" .. SPELL_BOOK_PROFESSIONS[professionIndex]["maxRank"])
					button.profText:SetPoint("LEFT", button, "RIGHT", BobUI_Settings["SpellIconSize"] + 8, 0);
					button.profText:Show()
					button.unlearn:SetScript("OnClick", function()
						StaticPopup_Show("UNLEARN_SKILL", SPELL_BOOK_PROFESSIONS[professionIndex]["name"], nil, SPELL_BOOK_PROFESSIONS[professionIndex]["skillLine"]);
					end)
				end

				if ((professionIndex == 1 or professionIndex == 2) and isFirstOfProfession) then
					button.unlearn:Show()
				end
				
				if isFirstOfProfession then
					LastFirstOfProfession = buttons[i]
				end
			end

			button:Show(); 
			

			if isPassive then j = j + 1; elseif spellID == nil and SPELL_BOOK_TYPE == "pet" then l = l + 1 elseif spellType == "FLYOUT" then m = m + 1 else k = k + 1 end
		end
	end
end

local waitTable = {};
local waitFrame = nil;

function wait(delay, func, ...)
  if(type(delay)~="number" or type(func)~="function") then
    return false;
  end
  if(waitFrame == nil) then
    waitFrame = CreateFrame("Frame","WaitFrame", UIParent);
    waitFrame:SetScript("onUpdate",function (self,elapse)
      local count = #waitTable;
      local i = 1;
      while(i<=count) do
        local waitRecord = tremove(waitTable,i);
        local d = tremove(waitRecord,1);
        local f = tremove(waitRecord,1);
        local p = tremove(waitRecord,1);
        if(d>elapse) then
          tinsert(waitTable,i,{d-elapse,f,p});
          i = i + 1;
        else
          count = count - 1;
          f(unpack(p));
        end
      end
    end);
  end
  tinsert(waitTable,{delay,func,{...}});
  return true;
end

function createSpellButtons()
	local buttons, button;
	local numFlyoutSpells = 0

	if ( BobTabPage1.buttons ) then
		buttons = BobTabPage1.buttons;
	else
		buttons = {}
	end

	local _, _, offset = GetSpellTabInfo(VIEWED_SPELL_BOOK);

	-- creates flyout buttons
	for i = 1, VIEWED_SPELL_BOOK_NUM_SPELLS do
		local spellType, actionID = GetSpellBookItemInfo(i+offset, SPELL_BOOK_TYPE);
		if spellType == "FLYOUT" then
			local numCurrentFlyoutSpells = select(3, GetFlyoutInfo(actionID))

			if numFlyoutSpells < numCurrentFlyoutSpells then
				numFlyoutSpells = numCurrentFlyoutSpells
			end
		end
	end

	local className, classTag, classID = UnitClass("player");
	local numSpecs = GetNumSpecializationsForClassID(classID);

	local numButtonsToCreate = 0

	for i=2, numSpecs+1 do
		if GetNumSpellsForSpellBook(i) > numButtonsToCreate then numButtonsToCreate = GetNumSpellsForSpellBook(i) end
	end

	-- create spell buttons
	for i = #buttons + 1, numButtonsToCreate + numFlyoutSpells do 

		button = CreateFrame("Button", "abilityButton"..i, BobTabPage1, "BobUI_SpellButtonTemplate,SecureActionButtonTemplate");
		button:SetSize(32,32)
		button:RegisterForDrag("LeftButton");

		tinsert(buttons, button)
	end
	
	BobTabPage1.buttons = buttons
end

function specButtonsUpdateBorders()

	local className, classTag, classID = UnitClass("player");
	local numSpecs = GetNumSpecializationsForClassID(classID);
	local currentSpecID = GetSpecialization()
	

	local lastShownButton = nil

	for i=1,8 do
		local button = BobUI_specButtons["specButton"..i];



		if (i == currentSpecID+1) then
			button.bg:SetColorTexture(getBackDropBorderColorFromClassColors(false))
		else
			if button:GetID() == VIEWED_TAB_ID then
				button.bg:SetColorTexture(getBackDropBorderColorFromClassColors(true))
			else
				button.bg:SetColorTexture(0.1, 0.1, 0.1, 1)
			end
		end

		if button:IsShown() then
			if lastShownButton ~= nil then
				button:SetPoint("TOPLEFT", lastShownButton, "TOPRIGHT", 5, 0)
			end

			lastShownButton = button
		else
			if i < 8 then
				local nextButton = _G["SpecButton" .. i+1]
				nextButton:SetPoint("TOPLEFT", lastShownButton, "TOPRIGHT", 5, 0)
			end
		end
	end

end

function BobUI_TalentFrameSpec_OnShow(self)
	self:RegisterEvent("PLAYER_SPECIALIZATION_CHANGED");
end
function BobUI_TalentFrameSpec_OnHide(self)
	self:UnregisterEvent("PLAYER_SPECIALIZATION_CHANGED");
end

function BobUI_TalentFrameSpec_OnEvent(self, event, ...)
	if event == "PLAYER_SPECIALIZATION_CHANGED" and VIEWED_SPELL_BOOK ~= nil then
		VIEWED_SPELL_BOOK = 2
		VIEWED_SPELL_BOOK_NUM_SPELLS = GetNumSpellsForSpellBook(VIEWED_SPELL_BOOK)

		createSpellButtons()
		updateSpellButtons()
		specButtonsUpdateBorders()
	end
end





function SpecButton_OnShow(self)
	local className, classTag, classID = UnitClass("player");
	local numSpecs = GetNumSpecializationsForClassID(classID);
	local currentSpecID = GetSpecialization()
	local cSpecID, currentSpecName = GetSpecializationInfo(currentSpecID)
	local sex = UnitSex("player");
	
	local roleNames = {
		["HEALER"] = HEALER,
		["DAMAGER"] = DAMAGER,
		["TANK"] = TANK
	}
	
	BobUI_TabTitle.text:SetText(SPECIALIZATION .. ": " .. currentSpecName)

	local numTabs = 2 + numSpecs + (PetHasSpellbook() and 1 or 0) -- general tab & crafting tab + num specs + pet spells?


	self:SetSize(BobUI_Settings["TabIconSize"] + (BobUI_Settings["BorderSize"] * 2), BobUI_Settings["TabIconSize"] + (BobUI_Settings["BorderSize"] * 2))
	self.specIcon:SetSize(BobUI_Settings["TabIconSize"], BobUI_Settings["TabIconSize"])
	self:Show();

	if self:GetID() == 1 then

			self.specIcon:SetTexture(136830)
			self.roleIcon:Hide();
			self.tooltip = GetSpellTabInfo(select(1, 1))

	elseif self:GetID() <= 1+numSpecs then
			local specID, specName, specDescription, specIcon, specRole, mainStat = GetSpecializationInfo(self:GetID()-1) 
			
			self.specIcon:SetTexture(specIcon);
			
			self.tooltip =  specName .. " " .. className ..  "\n\n" .. specDescription .. "\n\n" .. ROLE .. ": " .. roleNames[specRole];
			self.roleIcon:SetTexCoord(GetTexCoordsForRole(specRole));
	elseif self:GetID() > numSpecs and self:GetID() < 6 then
			self:Hide();
	
	elseif self:GetID() == 6 then
		local hasPetSpells, petToken = HasPetSpells();

		if ( hasPetSpells and PetHasSpellbook() ) then
			self.specIcon:SetTexture("Interface/Icons/ABILITY_HUNTER_SICKEM");
			self.roleIcon:Hide();
		else
			if VIEWED_SPELL_BOOK == 6 then VIEWED_SPELL_BOOK = 2 end
			self:Hide();
		end
	elseif self:GetID() == 7 then
			self.specIcon:SetTexture("Interface/Icons/TRADE_ARCHAEOLOGY_OGRE2HANDEDHAMMER");
			self.roleIcon:Hide();
			self.tooltip = TRADE_SKILLS
	elseif self:GetID() == 8 then
			self.specIcon:SetTexture("Interface/Icons/INV_Jewelry_Trinket_16");
			self.roleIcon:Hide();
			self.tooltip = TALENTS
	end

		VIEWED_SPELL_BOOK_NUM_SPELLS = GetNumSpellsForSpellBook(VIEWED_SPELL_BOOK)

		specButtonsUpdateBorders()

end

function SpecButton_OnDoubleClick(self) 
	local className, classTag, classID = UnitClass("player");
	local numSpecs = GetNumSpecializationsForClassID(classID);
	if self:GetID() < 2 or self:GetID() > numSpecs+1 or self:GetID()-1 == GetSpecialization() then return end -- can't change specialization to general spell book or pet/profession. 

	VIEWED_SPELL_BOOK = self:GetID()
	SetSpecialization(self:GetID()-1)
end

function SpecButton_OnClick(self)
	VIEWED_TAB_ID = self:GetID()
	
	if self:GetID() == 8 then
		BobUI_toggleTabPage(BobTabPage2)
	else
		BobUI_toggleTabPage(BobTabPage1)
		local className, classTag, classID = UnitClass("player");
		local numSpecs = GetNumSpecializationsForClassID(classID);

		specs = {}

		table.insert(specs, 1)

		for i=2, numSpecs+1 do -- current specialization is always number 2 spell book.. so need to rearrange books
			if (i < (GetSpecialization()+1)) then
				table.insert(specs, i + 1)
			elseif (i > (GetSpecialization()+1)) then
				table.insert(specs, i)
			else
				table.insert(specs, 2)
			end
		end

		if self:GetID() <= 5 then 
			VIEWED_SPELL_BOOK = specs[self:GetID()]
		else 
			VIEWED_SPELL_BOOK = self:GetID()
		end

		VIEWED_SPELL_BOOK_NUM_SPELLS = GetNumSpellsForSpellBook(VIEWED_SPELL_BOOK)
		CURRENT_FLYOUT = nil

		createSpellButtons()
		updateSpellButtons()
		updateSpellBookLabel()
	end

	specButtonsUpdateBorders()
	resizeBackground() 
end

function updateSpellBookLabel()
	if VIEWED_SPELL_BOOK < 6 then
		BobTabPage1Actives.category:SetText(GetSpellTabInfo(VIEWED_SPELL_BOOK))
	elseif VIEWED_SPELL_BOOK == 6 then
		BobTabPage1Actives.category:SetText(PET)
	else
		--BobTabPage1Actives.category:SetText(TRADE_SKILLS)
		BobTabPage1Actives.category:SetText()
	end
end

function SpecButton_OnEnter(self)
	if ( not self.selected ) then
		GameTooltip:SetOwner(self, "ANCHOR_TOP");
		GameTooltip:AddLine(self.tooltip, NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b);
		if self:GetID() > 1 and self:GetID() < 5 then
			GameTooltip:SetMinimumWidth(300, true);
		end

		GameTooltip:Show();
	end
end

function SpecButton_OnLeave(self)
	GameTooltip:SetMinimumWidth(0, false);
	GameTooltip:Hide();
end

function BobUI_AbilityTab_OnEvent(self, event)
	if not InCombatLockdown() then
		if event == "PLAYER_REGEN_DISABLED" then
			if BobUI_AbilityTab:IsShown() then
				SHOW_BOBUI_ABILITY_TAB_AFTER_COMBAT = true
				BobUI_AbilityTab:Hide()
			else
				SHOW_BOBUI_ABILITY_TAB_AFTER_COMBAT = false
			end
		elseif event == "PLAYER_REGEN_ENABLED" then
			if SHOW_BOBUI_ABILITY_TAB_AFTER_COMBAT then
			BobUI_AbilityTab:Show()
			end
		else
			
			if (event == "SPELLS_CHANGED") then
				SpecButton_OnShow(BobUI_specButtons.specButton6)
				createSpellButtons()
				updateSpellButtons()
				specButtonsUpdateBorders()
			end
		end
	end
end

function toggleSpellBookSettings()
	local isShown = BobUI_SettingsFrame:IsShown()

	if isShown then BobUI_SettingsFrame:Hide() else BobUI_SettingsFrame:Show() end
end

function CliqueShown()
	if CliqueConfig ~= nil then
		return CliqueConfig:IsShown()
	end

	return false
end


function BobUI_AbilityTab_OnLoad()
	tinsert(UISpecialFrames, "BobUI_AbilityTab")
	tinsert(UISpecialFrames, "BobUI_SettingsFrame")
	HideFrames:Hide()

	SpellBookFrame:SetParent(HideFrames)

	BobUI_AbilityTab:RegisterEvent("PLAYER_REGEN_DISABLED")
	BobUI_AbilityTab:RegisterEvent("PLAYER_REGEN_ENABLED")

	SpellBookFrame:SetScript("OnShow", function()
		if CliqueShown() == false and (HasPendingGlyphCast() == false and IsPendingGlyphRemoval() == false) then
			if SpellBookFrame:GetParent() ~= HideFrames then SpellBookFrame:SetParent(HideFrames) end
			SpellBookFrame:UnregisterAllEvents();
		end
	end)
	

	hooksecurefunc("TalentFrame_LoadUI", function()
		PlayerTalentFrame:SetParent(HideFrames)
		PlayerTalentFrame:UnregisterAllEvents();
	end)

	hooksecurefunc("ToggleSpellBook", function(bookType)
		if bookType == "professions" then
			if SPELL_BOOK_TYPE == "spell" then
				SpecButton7:Click()
				if BobUI_AbilityTab:IsShown() == false then toggleSpellBook() end
			end
		else
			if SPELL_BOOK_TYPE == "profession" then
				_G["SpecButton"..(GetSpecialization() + 1)]:Click()
				if BobUI_AbilityTab:IsShown() == false then toggleSpellBook() end
			end
			-- if shown professions, then go to specs spell book
		end
	end)

	hooksecurefunc("ShowUIPanel", function(frame)
		local frameName = frame
		if type(frame) == "table" then frameName = frame:GetName() end

		if frame ~= nil then
			if ((frameName == "SpellBookFrame" and CliqueShown() == false and (HasPendingGlyphCast() == false and IsPendingGlyphRemoval() == false)) or (frameName == "PlayerTalentFrame")) then
				if frame:IsShown() then HideUIPanel(frame) end
				toggleSpellBook()
			else
				if frameName == "SpellBookFrame" then
					SpellBookFrame:SetParent(UIParent)
					SpellBookFrame.bookType = BOOKTYPE_SPELL;
					SpellBookFrame_Update();
				end
			end
		end

	end)

	if VIEWED_SPELL_BOOK == nil then
		VIEWED_SPELL_BOOK = 2
		VIEWED_TAB_ID = GetSpecialization() + 1
	end
	
	BobUI_AbilityTab:SetFrameStrata("HIGH")
	BobUI_AbilityTab:SetFrameLevel(9)
	BobUI_SettingsFrame:SetFrameStrata("DIALOG")
	BobUI_SettingsFrame:SetFrameLevel(90)

	BobUI_toggleTabPage(BobTabPage1)

	

end


function setupColorButton(self)
	local colorButton = self


	self:SetSize(self:GetWidth() + 40, self:GetHeight() + 10)
	self.bg:SetSize(self:GetSize())

	colorButton.SetColorRGBA = function( self, newR, newG, newB, newA )
		if colorButton.Background then
			if colorButton.affectedFrames ~= nil then
				for k,v in pairs(colorButton.affectedFrames) do
					if colorButton.Separator then
						if _G[v].borderLeft ~= nil then
							_G[v].borderLeft:SetColorTexture(newR, newG, newB, newA)
						end
						if _G[v].borderRight ~= nil then
							_G[v].borderRight:SetColorTexture(newR, newG, newB, newA)
						end
						if _G[v].hSeparatorTop ~= nil then
							_G[v].hSeparatorTop:SetColorTexture(newR, newG, newB, newA)
						end
						if _G[v].hSeparatorBottom ~= nil then
							_G[v].hSeparatorBottom:SetColorTexture(newR, newG, newB, newA)
						end
					else
						if _G[v].bg ~= nil then
							_G[v].bg:SetColorTexture(newR, newG, newB, newA)
						end
					end
				end
			end
		else
			if colorButton.affectedFrames ~= nil then
				for k,v in pairs(colorButton.affectedFrames) do
					if _G[v].category ~= nil then
						_G[v].category:SetTextColor(newR, newG, newB, newA)
					end
					if _G[v].text ~= nil then
						_G[v].text:SetTextColor(newR, newG, newB, newA)
					end
				end
			end
		end
		
		if not ColorPickerFrame:IsShown() then
			if colorButton.Background then
				if colorButton.Separator then
					BobUI_Settings["SeparatorColor"] = {
						["r"] = newR, 
						["g"] = newG, 
						["b"] = newB, 
						["a"] = newA
					}
				else
					BobUI_Settings["BackgroundColor"] = {
						["r"] = newR, 
						["g"] = newG, 
						["b"] = newB, 
						["a"] = newA
					}
				end
			else
				BobUI_Settings["TextColor"] = {
					["r"] = newR, 
					["g"] = newG, 
					["b"] = newB, 
					["a"] = newA
				}
			end
		end
	end
	
	colorButton.cancelFunc = function()
		if colorButton.r ~= nil and colorButton.g ~= nil and colorButton.b ~= nil then

			colorButton:SetColorRGBA(ColorPickerFrame.previousValues.r, ColorPickerFrame.previousValues.g, ColorPickerFrame.previousValues.b, ColorPickerFrame.previousValues.opacity)
		
		end
	end

	
	colorButton.swatchFunc = function(info)
		if ColorPickerFrame:GetColorRGB() ~= nil then
			local r,g,b = ColorPickerFrame:GetColorRGB()

			colorButton:SetColorRGBA( r,g,b, 1 - OpacitySliderFrame:GetValue())
		end
	end
	
	colorButton:SetScript( "OnClick", function( self )
		if ColorPickerFrame:IsShown() then
			ColorPickerFrame:Hide()
		else
			self.r, self.g, self.b = r, g, b
			ColorPickerFrame:Hide()

			local info = UIDropDownMenu_CreateInfo();
			local colors = {}

			if colorButton.Background then
				if colorButton.Separator then
					if BobUI_Settings["SeparatorColor"] == nil then BobUI_Settings["SeparatorColor"] = BobUI_Settings_Recommended["SeparatorColor"] end
					colors = BobUI_Settings["SeparatorColor"]
				else
					if BobUI_Settings["BackgroundColor"] == nil then BobUI_Settings["BackgroundColor"] = BobUI_Settings_Recommended["BackgroundColor"] end
					colors = BobUI_Settings["BackgroundColor"]
				end
				
			else
				if BobUI_Settings["TextColor"] == nil then BobUI_Settings["TextColor"] = BobUI_Settings_Recommended["TextColor"] end
				colors = BobUI_Settings["TextColor"]
			end

			info.r, info.g, info.b = colors["r"], colors["g"], colors["b"]

			info.hasOpacity = 1
			info.opacity = 1 - colors["a"]

			info.swatchFunc = colorButton.swatchFunc
			info.func = colorButton.swatchFunc
			info.opacityFunc = colorButton.swatchFunc
			info.cancelFunc = colorButton.cancelFunc

			OpenColorPicker(info);
			ColorPickerFrame:SetPoint("TOPRIGHT", colorButton, "BOTTOMRIGHT", 0, -10)
			ColorPickerFrame:SetFrameStrata( "TOOLTIP" )
			ColorPickerFrame:Raise()
		end
	end )
end

function BobUI_toggleTabPage(shown)
	shown:Show()
	local hidden = nil

	if shown == BobTabPage1 then hidden = BobTabPage2 else hidden = BobTabPage1 end

	BobUI_PlayerTalentFrame:SetPoint("TOPLEFT", shown, "TOPRIGHT", 10, 0)

	if shown == BobTabPage2 then

		BobTabPage2.nameEditBox:SetPoint("TOPLEFT", BobTabPage2, "TOPLEFT", 4, -15)
		BobTabPage2.nameEditBox:SetSize((BobTabPage2NameEditBox.bg:GetWidth() - 4) + BobTabPage2ControlButton2:GetWidth(), BobTabPage2NameEditBox:GetHeight())
		
		BobTabPage2:SetWidth(BobTabPage2.nameEditBox:GetWidth() + 18 + BobTabPage2ScrollFrame:GetWidth() + 22)
		--BobTabPage2ControlButton2:SetPoint("TOPLEFT", BobTabPage2NameEditBox.bg, "TOPRIGHT", 2, 0)
	end
	hidden:Hide()
end

function resizeBackground() 
	if BobTabPage1:GetHeight() > BobUI_PlayerTalentFrame:GetHeight() then
		BobUI_AbilityTab.bg:SetPoint("BOTTOM", BobTabPage1, "BOTTOM", 0, -10)
	else
		BobUI_AbilityTab.bg:SetPoint("BOTTOM", BobUI_PlayerTalentFrame, "BOTTOM", 0, -10)
	end

	BobUI_AbilityTab.bg:SetPoint("RIGHT", (showHeartEssences() and BobUI_PlayerTalentFrameTalentsEssences or BobUI_PlayerTalentFrameTalentsBobUI_PvpTalentFrame), "RIGHT", 20, 0)
	if not showHeartEssences()  then 
		BobUI_PlayerTalentFrameTalentsBobUI_PvpTalentFrame.borderRight:Hide()
	end

	local frameWidth = (BobTabPage1:IsShown() and BobTabPage1:GetWidth() or BobTabPage2:GetWidth()) + 10 + BobUI_PlayerTalentFrame:GetWidth() + 10 + BobUI_PlayerTalentFrameTalentsBobUI_PvpTalentFrame:GetWidth() + (showHeartEssences() and BobUI_PlayerTalentFrameTalentsEssences:GetWidth() or 0) + 30
	
	
	BobUI_TabTitle.text:SetFont("Fonts/ARIALN.TTF", BobUI_Settings["FontSizeHeader"])

	local titleHeight = max(20, BobUI_Settings["FontSizeHeader"])

	BobUI_TabTitle:SetPoint("RIGHT", BobUI_AbilityTab, "LEFT", frameWidth, 0)
	BobUI_TabTitle:SetPoint("LEFT", BobUI_AbilityTab, "LEFT", 0, 0)
	BobUI_TabTitle:SetPoint("TOP", BobUI_AbilityTab, "TOP", 0, 0)
	BobUI_TabTitle:SetPoint("BOTTOM", BobUI_AbilityTab, "TOP", 0, -1 * (titleHeight + 30))

	BobUI_specButtons:SetHeight(BobUI_Settings["TabIconSize"] + (BobUI_Settings["BorderSize"] * 2) + (BobUI_Settings["SeparatorSize"] * 2) + 20)

	BobUI_specButtons.hSeparatorTop:SetPoint("TOP", BobUI_specButtons, "BOTTOM", 0, BobUI_Settings["SeparatorSize"])
	BobUI_specButtons.hSeparatorBottom:SetPoint("BOTTOM", BobUI_specButtons, "TOP", 0, -1 * BobUI_Settings["SeparatorSize"])

end


function BobUI_AbilityTab_OnShow(self)
	self:RegisterEvent("SPELLS_CHANGED");
	self:RegisterEvent("SPELL_UPDATE_COOLDOWN");
	self:RegisterEvent("UPDATE_SHAPESHIFT_FORM");
	self:RegisterEvent("CURRENT_SPELL_CAST_CHANGED");
	self:RegisterEvent("TRADE_SKILL_SHOW");
	self:RegisterEvent("TRADE_SKILL_CLOSE");
	self:RegisterEvent("ARCHAEOLOGY_CLOSED");
	self:RegisterEvent("PET_BAR_UPDATE");
	self:RegisterEvent("CURSOR_UPDATE");
	self:RegisterEvent("ACTIONBAR_SLOT_CHANGED");
	

	local prof1, prof2, arch, fish, cook = GetProfessions();
	BobUI_FormatProfession(1, prof1);
	updateSpellBookLabel()
	BobUI_AbilityTab:SetScale(BobUI_Settings["Scaling"])

	createSpellButtons()
	updateSpellButtons()

	resizeBackground() 

	MicroButtonPulseStop(TalentMicroButton);
	TalentMicroButtonAlert:Hide();
	UpdateMicroButtons();

	if BobUI_Settings["BackgroundColor"] == nil then BobUI_Settings["BackgroundColor"] = BobUI_Settings_Recommended["BackgroundColor"] end

	local backgroundColors = BobUI_Settings["BackgroundColor"]

	self.bg:SetColorTexture(backgroundColors["r"], backgroundColors["g"], backgroundColors["b"], backgroundColors["a"])

	local TextColors = BobUI_Settings["TextColor"]
	
	affectedFrames = {}

	table.insert(affectedFrames, "BobTabPage1Actives")
	table.insert(affectedFrames, "BobTabPage1Passives")
	table.insert(affectedFrames, "BobUI_PlayerTalentFrameTalentsTalents")
	table.insert(affectedFrames, "BobUI_PlayerTalentFrameTalentsBobUI_PvpTalentFramePVPTalents")
	table.insert(affectedFrames, "BobUI_PlayerTalentFrameTalentsEssencesTitle")
	table.insert(affectedFrames, "BobUI_TabTitle")

	for k, v in pairs(affectedFrames) do
		if _G[v].category ~= nil then
			_G[v].category:SetTextColor(TextColors["r"], TextColors["g"], TextColors["b"], TextColors["a"])
		end
		if _G[v].text ~= nil then
			_G[v].text:SetTextColor(TextColors["r"], TextColors["g"], TextColors["b"], TextColors["a"])
		end
	end

	BobTabPage2SetName:SetFont("Fonts\\ARIALN.ttf", BobUI_Settings["FontSizeBody"] - 2)
	BobTabPage2SetName:SetTextColor(TextColors["r"], TextColors["g"], TextColors["b"], TextColors["a"])

	BobTabPage2TalentSetInfo:SetFont("Fonts\\ARIALN.ttf", BobUI_Settings["FontSizeBody"] - 2)
	BobTabPage2TalentSetInfo:SetTextColor(TextColors["r"], TextColors["g"], TextColors["b"], TextColors["a"])
end

function BobUI_AbilityTab_OnHide(self)
	self:UnregisterEvent("SPELLS_CHANGED");
	self:UnregisterEvent("SPELL_UPDATE_COOLDOWN");
	self:UnregisterEvent("UPDATE_SHAPESHIFT_FORM");
	self:UnregisterEvent("CURRENT_SPELL_CAST_CHANGED");
	self:UnregisterEvent("TRADE_SKILL_SHOW");
	self:UnregisterEvent("TRADE_SKILL_CLOSE");
	self:UnregisterEvent("ARCHAEOLOGY_CLOSED");
	self:UnregisterEvent("PET_BAR_UPDATE");
	self:UnregisterEvent("CURSOR_UPDATE");
	self:UnregisterEvent("ACTIONBAR_SLOT_CHANGED");

	if SELECTED_HEART_ESSENCE_SLOT ~= nil then SELECTED_HEART_ESSENCE_SLOT:OnClick() end
	BobUI_specButtons.specButton6:Show()

end

function toggleSpellBook()
    if InCombatLockdown() then
		return
	else
		if BobUI_AbilityTab:IsShown() then BobUI_AbilityTab:Hide() else BobUI_AbilityTab:Show() end
	end
end


--[[ replace SpellBookFrame_OpenToPageForSlot
function SpellBookFrame_OpenToPageForSlot(slot, reason)

	local alreadyOpen = BobUI_AbilityTab:IsShown();

	if not alreadyOpen then
		ShowUIPanel(SpellBookFrame);
	end

	local _, _, offset = GetSpellTabInfo(VIEWED_SPELL_BOOK);
	local relativeSlot = slot - offset
	local button = _G["abilityButton" .. relativeSlot];

	if (slotType == "FLYOUT") then

	else
		if (reason == OPEN_REASON_PENDING_GLYPH) then
			button.AbilityHighlight:Show();
			button.AbilityHighlightAnim:Play();
		elseif (reason == OPEN_REASON_ACTIVATED_GLYPH) then
			button.AbilityHighlightAnim:Stop();
			button.AbilityHighlight:Hide();
			button.GlyphActivate:Show();
			button.GlyphIcon:Show();
			button.GlyphTranslation:Show();
			button.GlyphActivateAnim:Play();
		end
	end
end]]